@@ -27,6 +27,7 @@
 #include <IOKit/IOLib.h>
 #include <IOKit/IOService.h>
 #include <IOKit/IOPlatformExpert.h>
+#include <IOKit/pwr_mgt/RootDomain.h>
 #include <IOKit/platform/ApplePlatformExpert.h>
 
 #include <IOKit/usb/IOUSBRootHubDevice.h>
@@ -157,8 +158,6 @@ void AppleUSBOHCI::initForPM (IOPCIDevice *provider)
     //  Deal with CardBus USB cards.  Their provider will be a "IOCardBusDevice", as opposed to a "IOPCIDevice"
     //
     _onCardBus = (0 != provider->metaCast("IOCardBusDevice"));
-    if ( _onCardBus )
-        _unloadUIMAcrossSleep = true;
 
     //  Now, look at PCI cards.  Note that the onboard controller's provider is an IOPCIDevice so we cannot use that
     //  to distinguish between USB PCI cards and the on board controller.  Instead, we use the existence of the
@@ -169,13 +168,25 @@ void AppleUSBOHCI::initForPM (IOPCIDevice *provider)
 	if (provider->hasPCIPowerManagement() && (provider->enablePCIPowerManagement() == kIOReturnSuccess))
 	{
 	    _hasPCIPwrMgmt = true;
+            setProperty("Card Type","Built-in");
 	}
         else
         {
             USBError(1, "%s[%p]::start OHCI controller will be unloaded across sleep",getName(),this);
             _unloadUIMAcrossSleep = true;
+            setProperty("Card Type","PCI");
         }
     }
+    else
+    {
+        setProperty("Card Type","Built-in");
+    }
+    
+    if ( _onCardBus )
+    {
+        setProperty("Card Type","CardBus");
+        _unloadUIMAcrossSleep = true;
+    }
     
     // callPlatformFunction symbols
     _usb_remote_wakeup = OSSymbol::withCString("usb_remote_wakeup");
@@ -285,6 +296,7 @@ AppleUSBOHCI::setPowerState( unsigned long powerStateOrdinal, IOService* whatDev
     
     if ( powerStateOrdinal == kOHCISetPowerLevelSuspend ) 
     {
+
         if ( _unloadUIMAcrossSleep )
         {
             USBLog(3,"%s[%p] Unloading UIM for bus %d before going to sleep",getName(),this, _busNumber );
@@ -334,9 +346,10 @@ AppleUSBOHCI::setPowerState( unsigned long powerStateOrdinal, IOService* whatDev
     
     if ( powerStateOrdinal == kOHCISetPowerLevelRunning ) 
     {
+
         // If we were just idle suspended, we did not unload the UIM, so we need to check that here
         //
-        if ( _unloadUIMAcrossSleep && !_idleSuspend )
+        if ( !_uimInitialized )
         {
             // If we are inactive OR if we are a PC Card and we have been ejected, then we don't need to do anything here
             //
@@ -351,10 +364,11 @@ AppleUSBOHCI::setPowerState( unsigned long powerStateOrdinal, IOService* whatDev
 
                 USBLog(5, "%s[%p]: Re-loading UIM if necessary (%d)", getName(), this, _uimInitialized );
 
-                if ( !_uimInitialized )
-                    UIMInitializeForPowerUp();
+                // Initialize the hardware
+                //
+                UIMInitializeForPowerUp();
 
-                _ohciAvailable = true;										// tell the interrupt filter routine that we are on
+                _ohciAvailable = true;                          // tell the interrupt filter routine that we are on
 		_ohciBusState = kOHCIBusStateRunning;
                 if ( _rootHubDevice == NULL )
                 {
@@ -374,6 +388,13 @@ AppleUSBOHCI::setPowerState( unsigned long powerStateOrdinal, IOService* whatDev
         {
             USBLog(2, "%s[%p] setPowerState powering on USB", getName(), this);
 	
+			// at this point, interrupts are disabled, and we are waking up. If the Port Change interrupt is active
+			// then it is likely that we are responsible for the system issuing the wakeup
+			if (USBToHostLong(_pOHCIRegisters->hcInterruptStatus) & kOHCIHcInterrupt_RD)
+			{
+				IOLog("USB caused wake event (OHCI)\n");
+			}
+			
             _remote_wakeup_occurred = true;	//doesn't matter how we woke up
 
             if (_hasPCIPwrMgmt)
@@ -474,7 +495,7 @@ AppleUSBOHCI::SuspendUSBBus()
     hcControl = kOHCIFunctionalState_Suspend << kOHCIHcControl_HCFSPhase;
     if (_hasPCIPwrMgmt)
 	hcControl |= kOHCIHcControl_RWC | kOHCIHcControl_RWE;
-    _pOHCIRegisters->hcControl = USBToHostLong(hcControl);
+    _pOHCIRegisters->hcControl = HostToUSBLong(hcControl);
     IOSleep(3);	// wait 3 milliseconds for things to settle
     
     /*
